﻿
/*

	FCS App Reloader
	by Jesse R. Warden
	jesterxl@jessewarden.com
	http://www.jessewarden.com
	jesse@universalmind.com
	http://www.universalmind.com

	This is release under a Creative Commons license. 
   	More information can be found here:
    
   	http://creativecommons.org/licenses/by/2.5/
	
	v 1.0.0
	This app connects to your FCS server, and reloads
	an application instance.  The typical use case is to
	launch this app from ANT in Eclipse.
	
	The alternative is to:
	- toggle to the Communication App Inspector
	- click the app instance and click "reload"
	- if it's not there, type in it's name, and then click "load"
	- toggle back to Eclipse
	
	Now, you can just click once on your ANT task, and you're done!
	
	This API assumes Flash Communication Server 1.0 and Flash Player 6.
	I tested this in Flash Player 8, and the API should work
	in Flash Media Server 2 per the livedocs.
	
*/

import mx.utils.Delegate;

function init()
{
	Stage.align = "TL";
	Stage.scaleMode = "noScale";
	Stage.addListener(this);
	
	createTextField("debug_txt", 0, 0, 0, 100, 100);
	debug_txt.background = true;
	debug_txt.backgroundColor = 0xFFFFFF;
	debug_txt.border = true;
	debug_txt.borderColor = 0x000000;
	debug_txt.html = true;
	debug_txt.multiline = true;
	debug_txt.selectable = true;
	debug_txt.wordWrap = true;
	
	var fmt:TextFormat = new TextFormat();
	fmt.font = "Verdana";
	fmt.size = 11;
	debug_txt.setTextFormat(fmt);
	debug_txt.setNewTextFormat(fmt);
	onResize();
	
	debugHeader();
	debug("Loading config xml...");
	
	NETCONNECTION_CALL_SUCCESS = "NetConnection.Call.Success";
	CONFIG_XML_FILE = "fcs_app_reloader_config.xml";
	
	TIMEOUT_WIDTH = 100;
	TIMEOUT_HEIGHT = 12;
	
	config_xml = new XML();
	config_xml.ignoreWhite = true;
	config_xml.owner = this;
	config_xml.onLoad = Delegate.create(this, onConfigLoaded);
	config_xml.load(CONFIG_XML_FILE);
}

function onConfigLoaded(success:Boolean):Void
{
	debugHeader();
	if(success == true)
	{
		debug("Config XML loaded, parsing...");
		var at:Object 		= config_xml.firstChild.attributes
		theHost				= (at.host == null) ? "localhost" : at.host;
		thePort				= (at.port == null) ? "1111" : at.port;
		theAppName 			= at.appname;
		theUsername 		= at.username;
		thePassword			= at.password;
		theHangTime			= (at.hangtime == null) ? 3 : parseInt(at.hangtime);
		if(isNaN(theHangTime) || theHangTime < 0 || theHangTime == null) theHangTime = 3;
		
		var tabs:String = "\t\t\t\t";
		debug("<font color='#000066'><b>host</b></font>: " 			+ theHost);
		debug("<font color='#000066'><b>port</b></font>: " 			+ thePort);
		debug("<font color='#000066'><b>app name</b></font>: " 		+ theAppName);
		debug("<font color='#000066'><b>user</b></font>: " 			+ theUsername);
		debug("<font color='#000066'><b>password</b></font>: " 		+ thePassword);
		debug("<font color='#000066'><b>hang time</b></font>: " 	+ theHangTime);
		
		connect();
	}
	else
	{
		debug("Failed to find " + CONFIG_XML_FILE + ", aborting.");
		lightFuse();
	}
}


function connect()
{
	debugHeader();
	debug("Connecting to FCS Admin...");
	
	nc = new NetConnection();
	nc.owner = this;
	nc.onStatus = function(info)
	{
		if(info.code == "NetConnection.Connect.Success")
		{
			debugHeader();
			debug("Successfully connected!");
			this.owner.onConnected();
		}
		else
		{
			debugHeader();
			debug("<font color='#660000'><b>Problem connecting to FCS Admin, aborting.</b></font>");
			this.owner.lightFuse();
		}
	};
	nc.connect("rtmp://" + theHost + ":" + thePort + "/" + theAppName,
			   theUsername,
			   thePassword);
}

function onConnected()
{
	//getActiveInstances();
	reloadApp("flex2test");
}

function reloadApp(str:String)
{
	debugHeader();
	debug("Reloading app...");
	nc.call("reloadApp", 
				new Responder(this, 
							  reloadApp_result, 
							  reloadApp_fault),
				str);
}

function reloadApp_result(o)
{
	debugHeader();
	if(o.code == NETCONNECTION_CALL_SUCCESS)
	{
		debug("<font color='#006600'><b>App successfully rebooted, exiting.</b></font>");
	}
	else
	{
		debug("<font color='#660000'><b>Failed to reboot the app!</font></b>");
		debugProps(o);
		debugProps(o.level);
		debugProps(o.error);
		debugProps(o.code);
		debug(o.description);
		debugProps(o.description);
		debug("Exiting.");
	}
	
	//debugProps(o);
	lightFuse();
}

function reloadApp_fault(o)
{
	debugHeader();
	debug("reloadApp_fault");
	debugProps(o);
	lightFuse();
}

/*
function getActiveInstances()
{
	debugHeader();
	debug("getActiveInstances...");
	nc.call("getActiveInstances", 
			new Responder(this, 
						  getActiveInstances_result, 
						  getActiveInstances_fault));
}

function getActiveInstances_result(o)
{
	debugHeader();
	debug("getActiveInstances_result");
	if(o.code == NETCONNECTION_CALL_SUCCESS)
	{
		debug(o.data.length + " active instances.");
		debugProps(o.data);
	}
}

function getActiveInstances_fault(o)
{
	debugHeader();

	debug("getActiveInstances_fault");
	debugProps(o);
}
*/

function lightFuse()
{
	nc.onStatus = null;
	nc.close();
	nc = null;
	
	createEmptyMovieClip("timeout_mc", 1);
	timeout_mc.st = getTimer();
	timeout_mc.timeout = 3; // seconds
	timeout_mc.onEnterFrame = function()
	{
		var ct:Number = getTimer();
		var et:Number = ct - this.st;
		if(et > (this.timeout * 1000))
		{
			this._parent.boom();
		}
		else
		{
			var p:Number = et / (this.timeout * 1000);
			var w:Number = TIMEOUT_WIDTH * p;
			
		}
		
		
		timeout_mc.clear();
		timeout_mc.lineStyle(0, 0x000000);
		timeout_mc.lineTo(TIMEOUT_WIDTH, 0);
		timeout_mc.lineTo(TIMEOUT_WIDTH, TIMEOUT_HEIGHT);
		timeout_mc.lineTo(0, TIMEOUT_HEIGHT);
		timeout_mc.lineTo(0, 0);
		
		timeout_mc.moveTo(1, 1);
		timeout_mc.beginFill(0x666666);
		timeout_mc.lineTo(w, 1);
		timeout_mc.lineTo(w, TIMEOUT_HEIGHT - 1);
		timeout_mc.lineTo(1, TIMEOUT_HEIGHT - 1);
		timeout_mc.lineTo(1, 1);
		
		timeout_mc.endFill();
	};
	timeout_mc.onEnterFrame(); // draw immediately
	onResize();
}

function boom()
{
	delete timeout_mc.onEnterFrame;
	timeout_mc.removeMovieClip();
	fscommand("quit");
}

function onResize()
{
	debug_txt._x = 0;
	debug_txt._y = 0;
	debug_txt._width = Math.max(12, Stage.width);
	debug_txt._height = Math.max(12, Stage.height);
	
	var sx:Number = Math.max(6, Stage.width);
	var sy:Number = 6;
	timeout_mc._x = sx - (timeout_mc._width) - 6;
	timeout_mc._y = sy;
}

function debug(o)
{
	trace(o);
	debug_txt.htmlText += o + "<br />";
}

function debugHeader()
{
	//debug("---------------");
}

function debugProps(o)
{
	if(o instanceof Array)
	{
		var len:Number = o.length;
		for(var i:Number = 0; i<len; i++)
		{
			trace(o[i]);
		}
	}
	else
	{
		for(var p in o)
		{
			debug(p + ": " + o[p]);
		}
	}
}

init();

